package adapter;

import factory.AbstractList;
import java.util.List;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.util.ListIterator;
import java.util.Iterator;
import java.util.ConcurrentModificationException;

public class ListAdapter
extends AbstractList
implements List
{
	Vector storage = new Vector();

	public ListIterator listIterator(int index)
	{
		ListAdapterListIterator retval = new ListAdapterListIterator();
		if(index < 0 || index > storage.size())
		{
			throw new NoSuchElementException();
		}
		retval.index = index;
		retval.list = this;
		return retval;
	}

	public int size()
	{
		return storage.size();
	}

	/*
	public ListIterator listIterator()
	{
		return listIterator(0);
	}
	*/

	public Object set(int index, Object element)
	{
		ListIterator li = listIterator(index);
		Object retval = li.next();
		li.set(element);

		lastVar = System.currentTimeMillis();
		lastManipulator = this;

		return retval;
	}

	protected List getConcreteInstance()
	{
		return new ListAdapter();
	}

	/*
	public Iterator iterator()
	{
		return listIterator(0);
	}
	*/

	class ListAdapterListIterator
	implements ListIterator
	{
		long dob = System.currentTimeMillis();
		protected ListAdapter list;
		int index = 0;
		int lastOp = 0;

		private void checkValidity()
		{
			//System.out.println("dob: " + dob + "; lastVar: " + list.lastVar);
			if(dob < list.lastVar)
			{
				if(!this.equals(list.lastManipulator))
				{
					throw new ConcurrentModificationException();
				}
			}
		}

		// Query Operations

		/**
		 * Returns <tt>true</tt> if this list iterator has more elements when
		 * traversing the list in the forward direction. (In other words, returns
		 * <tt>true</tt> if <tt>next</tt> would return an element rather than
		 * throwing an exception.)
		 *
		 * @return <tt>true</tt> if the list iterator has more elements when
		 *		traversing the list in the forward direction.
		 */
		public boolean hasNext()
		{
			checkValidity();
			return index < list.storage.size();
		}

		/**
		 * Returns the next element in the list.	This method may be called
		 * repeatedly to iterate through the list, or intermixed with calls to
		 * <tt>previous</tt> to go back and forth.	(Note that alternating calls
		 * to <tt>next</tt> and <tt>previous</tt> will return the same element
		 * repeatedly.)
		 *
		 * @return the next element in the list.
		 * @exception NoSuchElementException if the iteration has no next element.
		 */
		public Object next()
		{
			checkValidity();
			try
			{
				lastOp = 1;
				return list.storage.elementAt(index++);
			}
			catch(ArrayIndexOutOfBoundsException aioobe)
			{
				lastOp = 0;
				throw new java.util.NoSuchElementException(aioobe.toString());
			}
		}

		/**
		 * Returns <tt>true</tt> if this list iterator has more elements when
		 * traversing the list in the reverse direction.	(In other words, returns
		 * <tt>true</tt> if <tt>previous</tt> would return an element rather than
		 * throwing an exception.)
		 *
		 * @return <tt>true</tt> if the list iterator has more elements when
		 *				 traversing the list in the reverse direction.
		 */
		public boolean hasPrevious()
		{
			checkValidity();
			return index > 0;
		}

		/**
		 * Returns the previous element in the list.	This method may be called
		 * repeatedly to iterate through the list backwards, or intermixed with
		 * calls to <tt>next</tt> to go back and forth.	(Note that alternating
		 * calls to <tt>next</tt> and <tt>previous</tt> will return the same
		 * element repeatedly.)
		 *
		 * @return the previous element in the list.
		 * 
		 * @exception NoSuchElementException if the iteration has no previous
		 *						element.
		 */
		public Object previous()
		{
			checkValidity();
			try
			{
				lastOp = -1;
				return list.storage.elementAt(--index);
			}
			catch(ArrayIndexOutOfBoundsException aioobe)
			{
				lastOp = 0;
				throw new java.util.NoSuchElementException(aioobe.toString());
			}
		}

		/**
		 * Returns the index of the element that would be returned by a subsequent
		 * call to <tt>next</tt>. (Returns list size if the list iterator is at the
		 * end of the list.)
		 *
		 * @return the index of the element that would be returned by a subsequent
		 * 				 call to <tt>next</tt>, or list size if list iterator is at end
		 *				 of list. 
		 */
		public int nextIndex()
		{
			checkValidity();
			return index;
		}

		/**
		 * Returns the index of the element that would be returned by a subsequent
		 * call to <tt>previous</tt>. (Returns -1 if the list iterator is at the
		 * beginning of the list.)
		 *
		 * @return the index of the element that would be returned by a subsequent
		 * 				 call to <tt>previous</tt>, or -1 if list iterator is at
		 *				 beginning of list.
		 */ 
		public int previousIndex()
		{
			checkValidity();
			return index-1;
		}


		// Modification Operations
		
		/**
		 * Removes from the list the last element that was returned by
		 * <tt>next</tt> or <tt>previous</tt> (optional operation).	This call can
		 * only be made once per call to <tt>next</tt> or <tt>previous</tt>.	It
		 * can be made only if <tt>ListIterator.add</tt> has not been called after
		 * the last call to <tt>next</tt> or <tt>previous</tt>.
		 *
		 * @exception UnsupportedOperationException if the <tt>remove</tt>
		 * 			operation is not supported by this list iterator.
		 * @exception IllegalStateException neither <tt>next</tt> nor
		 *			<tt>previous</tt> have been called, or <tt>remove</tt> or
		 *			<tt>add</tt> have been called after the last call to *
		 *			<tt>next</tt> or <tt>previous</tt>.
		 */
		public void remove()
		{
			checkValidity();
			switch(lastOp)
			{
				case 0:// ne' next ne' previous->errore
					throw new IllegalStateException("Ne' next, ne' previous: lastOp = " + lastOp);
					//break;
				case 1://next
					list.storage.removeElementAt(--index);
					break;
				case -1://previous
					list.storage.removeElementAt(index);
					break;
				default://valore illegale di lastOp->abbiamo trovato un baco!
					throw new IllegalStateException("Valore illegale di lastOp = " + lastOp);
			}
			list.lastVar = System.currentTimeMillis();
			list.lastManipulator = this;
			lastOp = 0;
		}

		/**
		 * Replaces the last element returned by <tt>next</tt> or
		 * <tt>previous</tt> with the specified element (optional operation).
		 * This call can be made only if neither <tt>ListIterator.remove</tt> nor
		 * <tt>ListIterator.add</tt> have been called after the last call to
		 * <tt>next</tt> or <tt>previous</tt>.
		 *
		 * @param o the element with which to replace the last element returned by
		 *					<tt>next</tt> or <tt>previous</tt>.
		 * @exception UnsupportedOperationException if the <tt>set</tt> operation
		 * 			is not supported by this list iterator.
		 * @exception ClassCastException if the class of the specified element
		 * 			prevents it from being added to this list.
		 * @exception IllegalArgumentException if some aspect of the specified
		 *			element prevents it from being added to this list.
		 * @exception IllegalStateException if neither <tt>next</tt> nor
		 *						<tt>previous</tt> have been called, or <tt>remove</tt> or
		 *			<tt>add</tt> have been called after the last call to
		 * 			<tt>next</tt> or <tt>previous</tt>.
		 */
		public void set(Object o)
		{
			checkValidity();
			switch(lastOp)
			{
				case 0:// ne' next ne' previous->errore
					throw new IllegalStateException("Ne' next, ne' previous: lastOp = " + lastOp);
					//break;
				case 1://next
					list.storage.setElementAt(o, index-1);
					break;
				case -1://previous
					list.storage.setElementAt(o, index);
					break;
				default://valore illegale di lastOp->abbiamo trovato un baco!
					throw new IllegalStateException("Valore illegale di lastOp = " + lastOp);
			}
			lastOp = 0;
			list.lastVar = System.currentTimeMillis();
			list.lastManipulator = this;
		}

		/**
		 * Inserts the specified element into the list (optional operation).	The
		 * element is inserted immediately before the next element that would be
		 * returned by <tt>next</tt>, if any, and after the next element that
		 * would be returned by <tt>previous</tt>, if any.	(If the list contains
		 * no elements, the new element becomes the sole element on the list.)
		 * The new element is inserted before the implicit cursor: a subsequent
		 * call to <tt>next</tt> would be unaffected, and a subsequent call to
		 * <tt>previous</tt> would return the new element.	(This call increases
		 * by one the value that would be returned by a call to <tt>nextIndex</tt>
		 * or <tt>previousIndex</tt>.)
		 *
		 * @param o the element to insert.
		 * @exception UnsupportedOperationException if the <tt>add</tt> method is
		 * 			not supported by this list iterator.
		 * 
		 * @exception ClassCastException if the class of the specified element
		 * 			prevents it from being added to this list.
		 * 
		 * @exception IllegalArgumentException if some aspect of this element
		 *						prevents it from being added to this list.
		 */
		public void add(Object o)
		{
			checkValidity();
			list.storage.insertElementAt(o, index++);
			lastOp = 0;
			list.lastVar = System.currentTimeMillis();
			list.lastManipulator = this;
		}
	}
}
