package mediateca;

import java.io.*;
import java.util.*;

public class Mediateca
{
	Map<String, List<Media>> titoli;
	Map<String, List<Media>> tipi;
	Map<String, List<Media>> categorie;
	Map<String, Comparator<Media>> ordinamenti;
	BufferedReader stdin;
	PrintStream console;
	MediaFactory mf;

	public static void main(String[] argv) throws IOException
	{
		(new Mediateca()).exe(argv);
	}

	public Mediateca() throws IOException
	{
		mf = new MediaFactory();
		titoli = new HashMap<String, List<Media>>();
		tipi = new HashMap<String, List<Media>>();
		categorie = new HashMap<String, List<Media>>();

		ordinamenti = new HashMap<String, Comparator<Media>>();

		ordinamenti.put("ComparaMediaPerTitolo", new ComparaMediaPerTitolo());
		ordinamenti.put("ComparaMediaPerCategoria", new ComparaMediaPerCategoria());
		ordinamenti.put("ComparaMediaPerCategoriaETitolo", new ComparaMediaPerCategoriaETitolo());
	}

	public void exe(String[] argv)
	{
		stdin = new BufferedReader(new InputStreamReader(System.in));
		console = System.out;

		while(true)
		{
			System.out.print("FILE/STAMPA/EXIT ? > ");
			String cmd = null;
			try
			{
				cmd = stdin.readLine();
			}
			catch(IOException ioe)
			{
				ioe.printStackTrace();
				return;
			}
			if(cmd == null || cmd.equals("EXIT"))
			{
				break;
			}
			else if(cmd.equals("FILE"))
			{
				file();
			}
			else if(cmd.equals("STAMPA"))
			{
				stampa();
			}
			else
			{
				console.println("Comando <" + cmd + "> non riconosciuto");
			}
		}
	}

	void file()
	{
		console.print("Nome file? > ");
		String nome = null;
		try
		{
			nome = stdin.readLine();
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
			return;
		}
		BufferedReader fbr;
		try
		{
			fbr = new BufferedReader(new FileReader(nome));
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
			return;
		}

		try
		{
			List<Media> tmp = null;
			Media m = null;

			while( (m = mf.nextMedia(fbr)) != null)
			{
				System.out.println("***\n" + m + "\n***");
				/* per titolo */
				tmp = titoli.get(m.titolo);
				if(tmp == null)
				{
					tmp = new LinkedList<Media>();
					titoli.put(m.titolo, tmp);
				}
				tmp.add(m);

				/* per tipo */
				tmp = tipi.get(m.getClass().getName());
				if(tmp == null)
				{
					tmp = new LinkedList<Media>();
					tipi.put(m.getClass().getName(), tmp);
				}
				tmp.add(m);

				/* per categoria */
				tmp = categorie.get(m.categoria);
				if(tmp == null)
				{
					tmp = new LinkedList<Media>();
					categorie.put(m.categoria, tmp);
				}
				tmp.add(m);
			}
		}
		catch(IncompleteDataException ide)
		{
			ide.printStackTrace();
		}

		System.out.println("titoli\n" + titoli);
		System.out.println("tipi\n" + tipi);
		System.out.println("categorie\n" + categorie);

		try
		{
			fbr.close();
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}
	}

	void stampa()
	{
		String tipoMedia = null;
		String tipoOrd = null;
		
		try
		{
			console.print("Tipo media? " + tipi.keySet() + " > ");
			tipoMedia = stdin.readLine();
			console.print("Tipo ordinamento? " + ordinamenti.keySet() + " > ");
			tipoOrd = stdin.readLine();
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
			return;
		}
		Comparator<Media> ord = ordinamenti.get(tipoOrd);
		if(ord == null)
		{
			console.println("Ordinamento <" + tipoOrd + "> non disponibile");
			return;
		}

		List<Media> list = tipi.get(tipoMedia);
		if(list == null)
		{
			console.println("Tipo media <" + tipoMedia + "> non disponibile");
			return;
		}

		Collections.sort(list, ord);
		console.println(list);
	}
}
