package trees;

import java.util.*;

public class NodoBinario<E>
{
	E value;
	NodoBinario<E> left, right;

	public NodoBinario(E val)
	{
		left = right = null;
		value = val;
	}

	public E getValue()
	{
		return value;
	}

	public NodoBinario<E> setRight(NodoBinario<E> r)
	{
		NodoBinario<E> retval = right;
		right = r;
		return retval;
	}

	public NodoBinario<E> setLeft(NodoBinario<E> l)
	{
		NodoBinario<E> retval = left;
		left = l;
		return retval;
	}

	public NodoBinario<E> getRight()
	{
		return right;
	}

	public NodoBinario<E> getLeft()
	{
		return left;
	}

	public String toStringPreOrder()
	{
		String retval = String.valueOf(value);
		retval = retval.concat("; ");
		if(left != null)
		{
			retval = retval.concat(left.toStringPreOrder());
		}
		retval = retval.concat("; ");
		if(right != null)
		{
			retval = retval.concat(right.toStringPreOrder());
		}
		retval = retval.concat("; ");
		return retval;
	}

	/**
	* Versione che usa null come marker di fine livello
	public String toStringBreadthFirst()
	{
		String retval = "";
		Queue<NodoBinario<E>> q = new LinkedList<NodoBinario<E>>();
		q.add(this);
		q.add(null);
		while(q.size() > 0)
		{
			NodoBinario<E> tmp = q.remove();
			if(tmp == null)
			{
				retval = retval.concat("\n");
				continue;
			}
			if(tmp.getLeft() != null)
			{
				q.add(tmp.getLeft());
			}
			if(tmp.getRight() != null)
			{
				q.add(tmp.getRight());
			}
			if(q.element() == null)
			{
				q.add(null);
			}
			retval = retval.concat(tmp.toString() + "; ");
		}
		return retval;
	}
	*/

	/**
	* Versione che usa un'istanza di NodoBinarioFinto come marker di fine livello
	*/
	public String toStringBreadthFirst()
	{
		String retval = "";
		Queue<NodoBinario<E>> q = new LinkedList<NodoBinario<E>>();
		q.add(this);
		q.add(new NodoBinarioFinto<E>());
		System.out.println(q);
		while(q.size() > 0)
		{
			NodoBinario<E> tmp = q.remove();
			if(tmp != null)
			{
				if(tmp instanceof NodoBinarioFinto)
				{
					retval = retval.concat("\n");
					continue;
				}
				//if(tmp.getLeft() != null)
				{
					q.add(tmp.getLeft());
				}
				//if(tmp.getRight() != null)
				{
					q.add(tmp.getRight());
				}
			}
			if(q.element() instanceof NodoBinarioFinto && !(q.size() == 1))
			{
				q.add(new NodoBinarioFinto<E>());
				System.out.println(q);
			}
			retval = retval.concat(String.valueOf(tmp) + "; ");
		}
		return retval;
	}

	public String toStringInOrder()
	{
		String retval = "";
		retval = retval.concat("(");
		if(left != null)
		{
			retval = retval.concat(left.toStringInOrder());
		}
		retval = retval.concat(" " + String.valueOf(value) + " ");
		if(right != null)
		{
			retval = retval.concat(right.toStringInOrder());
		}
		retval = retval.concat(")");
		return retval;
	}

	public String toStringPostOrder()
	{
		String retval = "";
		if(left != null)
		{
			retval = retval.concat(left.toStringInOrder());
		}
		retval = retval.concat("; ");
		if(right != null)
		{
			retval = retval.concat(right.toStringInOrder());
		}
		retval = retval.concat("; ");
		retval = retval.concat(String.valueOf(value) + "; ");
		return retval;
	}

	public Object evaluate()
	{
		if( !(value instanceof Operator) && (left != null || right != null))
		{
			System.err.println("Value non e' un operatore ma il nodo non e' una foglia");
			throw new IllegalStateException();
		}

		if(value instanceof Operator)
		{
			return ((Operator)value).evaluate(left.evaluate(), right.evaluate());
		}
		else
		{
			return value;
		}
	}

	public String toString()
	{
		return String.valueOf(value);
	}
}

